package com.fcbox.bluetooth.core;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;

/**
 * This class does all the work for setting up and managing Bluetooth connections with other
 * devices. It has a thread that listens for incoming connections, a thread for connecting with a
 * device, and a thread for performing data transmissions when connected.
 */
public class BluetoothService {

  // Debugging
  private static final String TAG = "BluetoothService";
  private static final boolean DEBUG = true;

  // Name for the SDP record when creating server socket
  private static final String NAME = "Printer";
  //UUID must be this
  // Unique UUID for this application
  private static final UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

  // Member fields
  private BluetoothAdapter mAdapter;
  private Handler mHandler;
  //    private AcceptThread mAcceptThread;
  private ConnectThread mConnectThread;
  private ConnectedThread mConnectedThread;
  private int mState;
  private int mNewState;

  // Constants that indicate the current connection state
  public static final int STATE_NONE = 0;       // we're doing nothing
  public static final int STATE_LISTEN = 1;     // now listening for incoming connections
  public static final int STATE_CONNECTING = 2; // now initiating an outgoing connection
  public static final int STATE_CONNECTED = 3;  // now connected to a remote device

  private BluetoothDevice connectedDevice;


  public BluetoothService(Handler handler) {
    mAdapter = BluetoothAdapter.getDefaultAdapter();
    mState = STATE_NONE;
    mNewState = mState;
    mHandler = handler;
  }

  private synchronized void updateToUserInterface() {

    mState = getState();
    if (DEBUG) {
      Log.d(TAG, "updateToUserInterface() " + mNewState + " -> " + mState);
    }

    mNewState = mState;
    mHandler.obtainMessage(BlueToothConstant.MESSAGE_STATE_CHANGE, mNewState, -1).sendToTarget();
  }

  public synchronized int getState() {
    return mState;
  }

  /**
   * Start the service. Specifically start AcceptThread to begin a session in listening (server)
   * mode. Called by the Activity onResume()
   */
  public synchronized void start() {
    if (DEBUG) {
      Log.d(TAG, "start");
    }

    if (mConnectThread != null) {
      mConnectThread.cancel();
      mConnectThread = null;
    }

    if (mConnectedThread != null) {
      mConnectedThread.cancel();
      mConnectedThread = null;
    }

//        if (mAcceptThread == null) {
//            mAcceptThread = new AcceptThread();
//            mAcceptThread.start();
//        }

    updateToUserInterface();
  }

  /**
   * Start the ConnectThread to initiate a connection to a remote device.
   *
   * @param device The BluetoothDevice to connect
   */
  public synchronized void connect(BluetoothDevice device) {
    if (DEBUG) {
      Log.d(TAG, "connect to: " + device);
    }

    // Cancel any thread attempting to make a connection
    if (mState == STATE_CONNECTING) {
      if (mConnectThread != null) {
        mConnectThread.cancel();
        mConnectThread = null;
      }
    }

    // Cancel any thread currently running a connection
    if (mConnectedThread != null) {
      mConnectedThread.cancel();
      mConnectedThread = null;
    }

    // Start the thread to connect with the given device
    mConnectThread = new ConnectThread(device);
    mConnectThread.start();

    updateToUserInterface();
  }

  /**
   * Start the ConnectedThread to begin managing a Bluetooth connection
   *
   * @param socket The BluetoothSocket on which the connection was made
   * @param device The BluetoothDevice that has been connected
   */
  public synchronized void connected(BluetoothSocket socket, BluetoothDevice device) {
    if (DEBUG) {
      Log.d(TAG, "connected");
    }

    // Cancel the thread that completed the connection
    if (mConnectThread != null) {
      mConnectThread.cancel();
      mConnectThread = null;
    }

    // Cancel any thread currently running a connection
    if (mConnectedThread != null) {
      mConnectedThread.cancel();
      mConnectedThread = null;
    }

    // Cancel the accept thread because we only want to connect to one device
//        if (mAcceptThread != null) {
//            mAcceptThread.cancel();
//            mAcceptThread = null;
//        }

    // Start the thread to manage the connection and perform transmissions
    mConnectedThread = new ConnectedThread(socket);
    mConnectedThread.start();

    // Send the name of the connected device back to the UI Activity
    Message msg = mHandler.obtainMessage(BlueToothConstant.MESSAGE_DEVICE_NAME);
    Bundle bundle = new Bundle();
    bundle.putString(BlueToothConstant.DEVICE_NAME, device.getName());
    bundle.putString(BlueToothConstant.DEVICE_ADDRESS, device.getAddress());
    msg.setData(bundle);
    mHandler.sendMessage(msg);

    connectedDevice = device;

    updateToUserInterface();
  }

  /**
   * Stop all threads
   */
  public synchronized void stop() {
    if (DEBUG) {
      Log.d(TAG, "stop");
    }

    if (mConnectThread != null) {
      mConnectThread.cancel();
      mConnectThread = null;
    }
    if (mConnectedThread != null) {
      mConnectedThread.cancel();
      mConnectedThread = null;
    }
//        if (mAcceptThread != null) {
//            mAcceptThread.cancel();
//            mAcceptThread = null;
//        }

    // Send a failure message back to the Activity

    if(connectedDevice!=null) {
      Message msg = mHandler.obtainMessage(BlueToothConstant.MESSAGE_DISCONNECT);
      Bundle bundle = new Bundle();
      bundle.putString(BlueToothConstant.DEVICE_NAME, connectedDevice.getName());
      bundle.putString(BlueToothConstant.DEVICE_ADDRESS, connectedDevice.getAddress());
      msg.setData(bundle);
      mHandler.sendMessage(msg);
    }

    connectedDevice = null;

    mState = STATE_NONE;
    updateToUserInterface();
  }

  public synchronized BluetoothDevice getConnectedDevice() {
    return connectedDevice;
  }

  public synchronized void clearConnectedDevice() {
    connectedDevice = null;
  }

  /**
   * Write to the ConnectedThread in an unsynchronized manner
   *
   * @param out The bytes to write
   * @see ConnectedThread#write(byte[])
   */
  public void write(byte[] out) {
    // Create temporary object
    ConnectedThread r;
    // Synchronize a copy of the ConnectedThread
    synchronized (this) {
      if (mState != STATE_CONNECTED) {
        return;
      }
      r = mConnectedThread;
    }
    r.write(out);
  }


  public boolean write(byte[] buffer, int offset, int length) {
    // Create temporary object
    ConnectedThread r;
    // Synchronize a copy of the ConnectedThread
    synchronized (this) {
      if (mState != STATE_CONNECTED) {
        return false;
      }
      r = mConnectedThread;
    }
    return r.write(buffer, offset, length);
  }


  private void connectionFailed(BluetoothDevice device) {
    // Send a failure message back to the Activity
    Message msg = mHandler.obtainMessage(BlueToothConstant.MESSAGE_UNABLE_CONNECT);
    Bundle bundle = new Bundle();
    bundle.putString(BlueToothConstant.DEVICE_NAME, device.getName());
    bundle.putString(BlueToothConstant.DEVICE_ADDRESS,device. getAddress());
    msg.setData(bundle);
    mHandler.sendMessage(msg);
    mState = STATE_NONE;
    updateToUserInterface();

    // Start the service over to restart listening mode
    BluetoothService.this.start();
  }


  private void connectionLost() {
    // Send a failure message back to the Activity
    Message msg = mHandler.obtainMessage(BlueToothConstant.MESSAGE_CONNECTION_LOST);
    mHandler.sendMessage(msg);
    mState = STATE_NONE;
    // Update UI title
    updateToUserInterface();

    // Start the service over to restart listening mode
    BluetoothService.this.start();
  }

  /**
   * This thread runs while listening for incoming connections. It behaves
   * like a server-side client. It runs until a connection is accepted
   * (or until cancelled).
   */
//    private class AcceptThread extends Thread {
//        // The local server socket
//        private final BluetoothServerSocket mmServerSocket;
//
//        public AcceptThread() {
//            BluetoothServerSocket tmp = null;
//
//            // Create a new listening server socket
//            try {
//                tmp = mAdapter.listenUsingRfcommWithServiceRecord(NAME, MY_UUID);
//            } catch (IOException e) {
//                Log.e(TAG, "listen() failed", e);
//            }
//            mmServerSocket = tmp;
//            mState = STATE_LISTEN;
//        }
//
//        @Override
//        public void run() {
//            if (DEBUG) Log.d(TAG, "BEGIN mAcceptThread" + this);
//            setName("AcceptThread");
//            BluetoothSocket socket = null;
//
//
//            // Listen to the server socket if we're not connected
//            while (mState != STATE_CONNECTED) {
//                try {
//                    // This is a blocking call and will only return on a
//                    // successful connection or an exception
//                    socket = mmServerSocket.accept();
//                } catch (Exception e) {
//                    Log.e(TAG, "accept() failed", e);
//                    break;
//                }
//
//                // If a connection was accepted
//                if (socket != null) {
//                    synchronized (BluetoothService.this) {
//                        switch (mState) {
//                            case STATE_LISTEN:
//                            case STATE_CONNECTING:
//                                // Situation normal. Start the connected thread.
//                                connected(socket, socket.getRemoteDevice());
//                                break;
//                            case STATE_NONE:
//                            case STATE_CONNECTED:
//                                // Either not ready or already connected. Terminate new socket.
//                                try {
//                                    socket.close();
//                                } catch (IOException e) {
//                                    Log.e(TAG, "Could not close unwanted socket", e);
//                                }
//                                break;
//                        }
//                    }
//                }
//            }
//            if (DEBUG) {
//                Log.i(TAG, "END mAcceptThread" + this);
//            }
//        }
//
//        public void cancel() {
//            if (DEBUG) {
//                Log.d(TAG, "cancel " + this);
//            }
//            try {
//                if (mmServerSocket != null) {
//                    mmServerSocket.close();
//                }
//            } catch (Exception e) {
//                Log.e(TAG, "close() of server failed", e);
//            }
//        }
//    }


  /**
   * This thread runs while attempting to make an outgoing connection with a device. It runs
   * straight through; the connection either succeeds or fails.
   */
  private class ConnectThread extends Thread {

    private final BluetoothSocket mmSocket;
    private final BluetoothDevice mmDevice;

    public ConnectThread(BluetoothDevice device) {
      mmDevice = device;
      BluetoothSocket tmp = null;

      // Get a BluetoothSocket for a connection with the
      // given BluetoothDevice
      try {
        tmp = device.createRfcommSocketToServiceRecord(MY_UUID);
      } catch (IOException e) {
        Log.e(TAG, "create() failed", e);
      }
      mmSocket = tmp;
      mState = STATE_CONNECTING;
    }

    @Override
    public void run() {
      Log.i(TAG, "BEGIN mConnectThread");
      setName("ConnectThread");

      // Always cancel discovery because it will slow down a connection
      mAdapter.cancelDiscovery();

      // Make a connection to the BluetoothSocket
      try {
        // This is a blocking call and will only return on a
        // successful connection or an exception
        if (mmSocket == null) {
          Log.e(TAG, "BluetoothSocket  is  null ");
          return;
        }
        mmSocket.connect();
      } catch (IOException e) {
        connectionFailed(mmDevice );
        // Close the socket
        try {
          mmSocket.close();
        } catch (IOException e2) {
          Log.e(TAG, "unable to close() socket during connection failure", e2);
        }
        // Start the service over to restart listening mode
        BluetoothService.this.start();
        return;
      }

      // Reset the ConnectThread because we're done
      synchronized (BluetoothService.this) {
        mConnectThread = null;
      }

      // Start the connected thread
      connected(mmSocket, mmDevice);
    }

    public void cancel() {
      try {
        mmSocket.close();
      } catch (Exception e) {
        Log.e(TAG, "close() of connect socket failed", e);
      }
    }
  }

  /**
   * This thread runs during a connection with a remote device. It handles all incoming and outgoing
   * transmissions.
   */
  private class ConnectedThread extends Thread {

    private final BluetoothSocket mmSocket;
    private final InputStream mmInStream;
    private final OutputStream mmOutStream;

    public ConnectedThread(BluetoothSocket socket) {
      Log.d(TAG, "create ConnectedThread");
      mmSocket = socket;
      InputStream tmpIn = null;
      OutputStream tmpOut = null;

      // Get the BluetoothSocket input and output streams
      try {
        tmpIn = socket.getInputStream();
        tmpOut = socket.getOutputStream();
      } catch (IOException e) {
        Log.e(TAG, "temp sockets not created", e);
      }

      mmInStream = tmpIn;
      mmOutStream = tmpOut;
      mState = STATE_CONNECTED;
    }

    @Override
    public void run() {
      Log.i(TAG, "BEGIN mConnectedThread");
      int bytes;

      // Keep listening to the InputStream while connected
      while (mState == STATE_CONNECTED) {
        Log.e(TAG, "ConnectedThread run");
        try {
          byte[] buffer = new byte[256];

          bytes = mmInStream.read(buffer);
          if (bytes > 0) {

            mHandler.obtainMessage(BlueToothConstant.MESSAGE_READ, bytes, -1, buffer)
                .sendToTarget();
          } else {
            Log.e(TAG, "disconnected bytes<=0 " + this.getName().toString());
            connectionLost();

            break;
          }
        } catch (IOException e) {
          Log.e(TAG, "disconnected IOException ");
          e.printStackTrace();

          connectionLost();

          break;
        }
      }
    }

    /**
     * Write to the connected OutStream.
     *
     * @param buffer The bytes to write
     */
    public void write(byte[] buffer) {
      try {
        mmOutStream.write(buffer);
        mmOutStream.flush();//清空缓存

        if (DEBUG) {
          Log.i(TAG, new String(buffer, "GBK"));
        }

        mHandler.obtainMessage(BlueToothConstant.MESSAGE_WRITE, -1, -1, buffer)
            .sendToTarget();
      } catch (IOException e) {
        Log.e(TAG, "Exception during write", e);
      }
    }

    // add for cpcl
    public boolean write(byte[] buffer, int offset, int length) {
      try {
        mmOutStream.write(buffer, offset, length);
        mmOutStream.flush();

        if (DEBUG) {
          Log.i(TAG, new String(buffer, "GBK"));
        }
//                mHandler.obtainMessage(BlueToothConstant.MESSAGE_WRITE, -1, -1, buffer)
//                        .sendToTarget();
        return true;
      } catch (Exception var5) {
        return false;
      }
    }


    public void cancel() {
      try {
        mmSocket.close();
      } catch (Exception e) {
        Log.e(TAG, "close() of connect socket failed", e);
      }
    }
  }
}
