package com.fcbox.bluetooth.core

import android.annotation.SuppressLint
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Handler
import android.os.Looper
import android.os.Message
import com.fcbox.bluetooth.bean.BlueToothBean
import java.util.*

/**
 * @Description: FcBluetoothManager
 * @Author: luobin
 * @Date:   2021/4/28
 * @License: Copyright Since 2020 Hive Box Technology. All rights reserved.
 * @Notice: This content is limited to the internal circulation of Hive Box, 
 * and it is prohibited to leak or used for other commercial purposes.
 */
class FcBluetoothManager private constructor() {

    private var mService: BluetoothService? = null
    var blueToothConnectCallBack: BlueToothConnectCallBack? = null
    private var blueToothDiscoveryCallBack: BlueToothDiscoveryCallBack? = null
    private var isInit = false

    private val discoveryReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            when (intent.action) {
                BluetoothDevice.ACTION_FOUND -> {
                    val device =
                        intent.getParcelableExtra<BluetoothDevice>(BluetoothDevice.EXTRA_DEVICE)
                    blueToothDiscoveryCallBack?.call(BluetoothDevice.ACTION_FOUND, device)
                }
                BluetoothAdapter.ACTION_DISCOVERY_FINISHED -> {
                    blueToothDiscoveryCallBack?.call(
                        BluetoothAdapter.ACTION_DISCOVERY_FINISHED, null
                    )
                }
            }
        }
    }

    fun init() {
        if (isInit) {
            return
        }
        mService = BluetoothService(mHandler)
        isInit = true
    }


//    fun start() {
//        if (mService == null) {
//            return
//        }
//        if (mService?.state == BluetoothService.STATE_NONE) {
//            mService?.start()
//        }
//    }

    fun connect(address: String?) {
        if (!BluetoothAdapter.checkBluetoothAddress(address)) {
            return
        }

        val mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter() ?: return
        val device = mBluetoothAdapter.getRemoteDevice(address)

        //close other, if have
        if (getState() != BluetoothService.STATE_NONE) {
            disConnect()
        }

        mService?.connect(device)

    }

    fun disConnect() {
        mService?.let {service->
            deviceList.clear()
            service.stop()
        }
    }

    fun stop() {
        disConnect()
        blueToothConnectCallBack = null
    }


    fun isBluetoothOpen(): Boolean {
        val mBluetoothAdapter = getDefaultAdapter()
        if (mBluetoothAdapter == null || !mBluetoothAdapter.isEnabled) {
            return false
        }
        return true
    }

    fun isConnected(): Boolean {
        return getState()==BluetoothService.STATE_CONNECTED
    }

    fun getDefaultAdapter(): BluetoothAdapter? {
        return BluetoothAdapter.getDefaultAdapter()
    }

    fun getState(): Int {
        return mService?.state ?: BluetoothService.STATE_NONE
    }


    fun getConnectedDevice(): BluetoothDevice? {
        return mService?.connectedDevice
    }


    fun clearConnectedDevice() {
        mService?.clearConnectedDevice()
    }

    fun sendDataByte(data: ByteArray?) {
        if (mService == null) {
            return
        }

        if (mService?.state != BluetoothService.STATE_CONNECTED) {
            return
        }

        mService?.write(data)
    }

    // add for cpcl
    fun sendDataByte(data: ByteArray?, offset: Int, length: Int): Boolean {
        if (mService == null) {
            return false
        }

        if (mService?.state != BluetoothService.STATE_CONNECTED) {
            return false
        }

        return mService!!.write(data, offset, length)
    }

    @SuppressLint("HandlerLeak")
    private val mHandler: Handler = object : Handler(Looper.getMainLooper()) {
        override fun handleMessage(msg: Message) {
            when (msg.what) {
                BlueToothConstant.MESSAGE_DEVICE_NAME -> {
                    val data = msg.data
                    val address = data.getString(BlueToothConstant.DEVICE_ADDRESS) ?: ""

                    if (address.isNotEmpty() && !deviceList.containsKey(address)) {
                        val name = data.getString(BlueToothConstant.DEVICE_NAME, "")
                        deviceList[address] = name
                    }
                }

                BlueToothConstant.MESSAGE_CONNECTION_LOST,
                BlueToothConstant.MESSAGE_UNABLE_CONNECT,
                BlueToothConstant.MESSAGE_BLUETOOTH_OFF -> deviceList.clear()
            }


            blueToothConnectCallBack?.call(
                CallBackMsg.newInstance(msg.what, msg.arg1, msg.arg2, msg.obj, msg.data)
            )
        }
    }

    fun getConnectedBlueList(): ArrayList<BlueToothBean> {
        val result = ArrayList<BlueToothBean>()
        if (!isBluetoothOpen()) {
            return result
        }

        for ((key, value) in deviceList) {
            val bean = BlueToothBean()
            bean.address = key
            bean.name = value
            bean.connectState = BlueToothBean.STATE_CONNECT_SUCCESS
            result.add(bean)
        }

        return result
    }

    fun startDiscovery(
        context: Context,
        discoveryCallBack: BlueToothDiscoveryCallBack?
    ) {
        stopDiscovery(context)
        blueToothDiscoveryCallBack = discoveryCallBack
        doDiscovery(context)
    }

    fun stopDiscovery(context: Context) {
        blueToothDiscoveryCallBack = null
        unregisterReceiver(context)
        getDefaultAdapter()?.cancelDiscovery()
    }

    private fun doDiscovery(context: Context) {
        registerReceiver(context)
        getDefaultAdapter()?.startDiscovery()
    }

    private fun registerReceiver(context: Context) {
        val filter = IntentFilter(BluetoothDevice.ACTION_FOUND)
        filter.addAction(BluetoothAdapter.ACTION_DISCOVERY_FINISHED)
        context.registerReceiver(discoveryReceiver, filter)
    }

    private fun unregisterReceiver(context: Context) {
        try {
            context.unregisterReceiver(discoveryReceiver)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    companion object {
        const val TAG = "FcBluetoothManager"
        private var INSTANCE: FcBluetoothManager = FcBluetoothManager()
        private val deviceList = LinkedHashMap<String, String>()

        fun getInstance(): FcBluetoothManager {
            return INSTANCE
        }
    }
}