package com.fcbox.printer.adapter

import android.graphics.Color
import com.fcbox.printer.base.IPrintAdapter
import com.fcbox.printer.opt.*
import com.fcbox.template.BaseTemplate
import com.fcbox.template.IPrintOpt
import java.util.*
import kotlin.experimental.or

/**
 * @Description: 默认cpcl适配器
 * @Author: luobin
 * @Date:   2021/4/26
 * @License: Copyright Since 2020 Hive Box Technology. All rights reserved.
 * @Notice: This content is limited to the internal circulation of Hive Box, 
 * and it is prohibited to leak or used for other commercial purposes.
 */
class DefaultCPCLPrintAdapter : IPrintAdapter {

    override fun parse(opts: List<IPrintOpt>): String {
        if (opts.isEmpty()) {
            return ""
        }

        var isBold = false
        val buffer = StringBuffer(1024 * 64)

        for (i in opts.indices) {
            val optCode = opts[i].optCode
            when (optCode) {
                OptTable.OPT_CODE_READY -> append(buffer, "! 0 200 200 840 1")

                OptTable.OPT_CODE_OVER -> {
                    append(buffer, "SETBOLD 0")
                    append(buffer, "GAP-SENSE")
                    append(buffer, "FORM")
                    append(buffer, "PRINT")
                }

                OptTable.OPT_CODE_TEXT -> {
                    val textOpt = opts[i] as TextOpt
                    if (textOpt.isBold() != isBold) {
                        isBold = textOpt.isBold()
                        if (isBold) {
                            append(buffer, "SETBOLD 1")
                        } else {
                            append(buffer, "SETBOLD 0")
                        }
                    }

                    val builder = StringBuilder()
                    builder.append("TEXT")
                    builder.append(" ")
                    builder.append(textOpt.getFontSizeMode())
                    append(
                        buffer, String.format(
                            "%s %d %d %s",
                            builder.toString(),
                            textOpt.getX().toInt(),
                            textOpt.getY().toInt(),
                            textOpt.getContent()
                        )
                    )
                }

                OptTable.OPT_CODE_LINE -> {
                    val lineOpt = opts[i] as LineOpt
                    append(
                        buffer, String.format(
                            "LINE %d %d %d %d %d",
                            lineOpt.getBeginX().toInt(),
                            lineOpt.getBeginY().toInt(),
                            lineOpt.getEndX().toInt(),
                            lineOpt.getEndY().toInt(),
                            lineOpt.getWidth()
                        )
                    )
                }

                OptTable.OPT_CODE_BARCODE -> {
                    val barCodeOpt = opts[i] as BarCodeOpt
                    var cpclRatio = 1
                    if (barCodeOpt.getRatio() == 1.5) {
                        cpclRatio = 0
                    }
                    append(
                        buffer, String.format(
                            "B 128 %d %d %d %d %d %s",
                            barCodeOpt.getWidth(),
                            cpclRatio,
                            barCodeOpt.getHeight(),
                            barCodeOpt.getX().toInt(),
                            barCodeOpt.getY().toInt(),
                            barCodeOpt.getNumber()
                        )
                    )
                }

                OptTable.OPT_CODE_QRCODE -> {
                    val qrCodeOpt = opts[i] as QrCodeOpt
                    append(
                        buffer, String.format(
                            "B QR %d %d M %d U %d\nMA,%s\nENDQR",
                            qrCodeOpt.getX().toInt(),
                            qrCodeOpt.getY().toInt(),
                            qrCodeOpt.getQrVersion(),
                            qrCodeOpt.getUnitHeight(),
                            qrCodeOpt.getContent()
                        )
                    )
                }

                OptTable.OPT_CODE_BITMAP -> {
                    val bitmapOpt = opts[i] as BitmapOpt
                    val byteWidth = (bitmapOpt.getBitmapWidth() - 1) / 8 + 1
                    val bitmapHeight = bitmapOpt.getBitmapHeight()
                    var bitmapData = ""
                    val dataByte = ByteArray(byteWidth * bitmapHeight)

                    for (h in 0 until bitmapOpt.getBitmapHeight()) {
                        for (w in 0 until bitmapOpt.getBitmapWidth()) {
                            if (bitmapOpt.getBmp()!!.getPixel(w, h) == Color.BLACK) {
                                dataByte[h * byteWidth + w / 8] =
                                    dataByte[h * byteWidth + w / 8] or (0x80 shr w % 8).toByte()
                            }
                        }
                    }

                    bitmapData = bitmapOpt.transToHexString(dataByte)
                    append(
                        buffer, String.format(
                            "EG %d %d %d %d %s",
                            byteWidth,
                            bitmapHeight,
                            bitmapOpt.getX().toInt(),
                            bitmapOpt.getY().toInt(),
                            bitmapData
                        )
                    )
                }

                OptTable.OPT_CODE_BOX -> {
                    val boxOpt = opts[i] as BoxOpt
                    append(
                        buffer, String.format(
                            "BOX %d %d %d %d %d",
                            boxOpt.getBeginX().toInt(),
                            boxOpt.getBeginY().toInt(),
                            boxOpt.getEndX().toInt(),
                            boxOpt.getEndY().toInt(),
                            boxOpt.getWidth()
                        )
                    )
                }

                OptTable.OPT_ALIGIN -> {
                    val alignOpt = opts[i] as AlignOpt
                    append(buffer, alignOpt.getType())
                }

            }
        }
        return buffer.toString()
    }

    private fun append(buffer: StringBuffer, content: String) {
        buffer.append(content)
        buffer.append("\n")
    }

    override fun adapterPix(mmIprintOpt: List<IPrintOpt>): List<IPrintOpt> {
        val pixPrintOptList: MutableList<IPrintOpt> =
            ArrayList()
        for (i in mmIprintOpt.indices) {
            val optCode = mmIprintOpt[i].optCode
            when (optCode) {
                OptTable.OPT_CODE_TEXT -> {
                    val textOpt = mmIprintOpt[i] as TextOpt
                    textOpt.setX(getValueByRatio(textOpt.getX()).toDouble())
                    textOpt.setY(getValueByRatio(textOpt.getY()).toDouble())
                    pixPrintOptList.add(textOpt)
                }
                OptTable.OPT_CODE_LINE -> {
                    val lineOpt = mmIprintOpt[i] as LineOpt
                    lineOpt.setBeginX(getValueByRatio(lineOpt.getBeginX()).toDouble())
                    lineOpt.setBeginY(getValueByRatio(lineOpt.getBeginY()).toDouble())
                    lineOpt.setEndX(getValueByRatio(lineOpt.getEndX()).toDouble())
                    lineOpt.setEndY(getValueByRatio(lineOpt.getEndY()).toDouble())
                    pixPrintOptList.add(lineOpt)
                }
                OptTable.OPT_CODE_BARCODE -> {
                    val barCodeOpt = mmIprintOpt[i] as BarCodeOpt
                    barCodeOpt.setX(getValueByRatio(barCodeOpt.getX()).toDouble())
                    barCodeOpt.setY(getValueByRatio(barCodeOpt.getY()).toDouble())
                    pixPrintOptList.add(barCodeOpt)
                }
                OptTable.OPT_CODE_QRCODE -> {
                    val qrCodeOpt = mmIprintOpt[i] as QrCodeOpt
                    qrCodeOpt.setX(getValueByRatio(qrCodeOpt.getX()).toDouble())
                    qrCodeOpt.setY(getValueByRatio(qrCodeOpt.getY()).toDouble())
                    pixPrintOptList.add(qrCodeOpt)
                }
                OptTable.OPT_CODE_BITMAP -> {
                    val bitmapOpt = mmIprintOpt[i] as BitmapOpt
                    bitmapOpt.setX(getValueByRatio(bitmapOpt.getX()).toDouble())
                    bitmapOpt.setY(getValueByRatio(bitmapOpt.getY()).toDouble())
                    pixPrintOptList.add(bitmapOpt)
                }
                OptTable.OPT_CODE_BOX -> {
                    val boxOpt = mmIprintOpt[i] as BoxOpt
                    boxOpt.setBeginX(getValueByRatio(boxOpt.getBeginX()).toDouble())
                    boxOpt.setBeginY(getValueByRatio(boxOpt.getBeginY()).toDouble())
                    boxOpt.setEndX(getValueByRatio(boxOpt.getEndX()).toDouble())
                    boxOpt.setEndY(getValueByRatio(boxOpt.getEndY()).toDouble())
                    pixPrintOptList.add(boxOpt)
                }
                else -> pixPrintOptList.add(mmIprintOpt[i])
            }
        }
        return pixPrintOptList
    }

    private fun getValueByRatio(mmValue: Double): Int {
        return (mmValue * BaseTemplate.UNIT_RATIO).toInt()
    }
}